/////////////////////////////////////////////////////////////////
//
//      Subdivision Geometry Shader Example
//      Author: Jag Mohan Singh
//      jagmohan [at] research [dot] iiit [dot] ac [dot] in
//      Center for Visual Information Technology
//      International Institute of Information Technology Hyderabad
//      http://cvit.iiit.ac.in/
//
/////////////////////////////////////////////////////////////////

#version 120
varying vec4 outposition;
uniform mat4 gx1;
uniform mat4 gy1;
uniform mat4 gz1;


vec4 floatposition( float s, float t, mat4 gx, mat4 gy, mat4 gz )
{

        float s2 = s*s;
        float s3 = s2*s;
        float t2 = t*t;
      	float t3 = t2*t;
        vec4 svec = vec4(s3,s2,s,1.0);
        vec4 tvec = vec4(t3,t2,t,1.0);

        vec4 xtemp;
        vec4 ytemp;
        vec4 ztemp;
	xtemp = svec*gx;
        ytemp = svec*gy;
        ztemp = svec*gz;
	float x1 = dot(xtemp,tvec);
        float y1 = dot(ytemp,tvec);
        float z1 = dot(ztemp,tvec);
        vec4 result = vec4(x1,y1,z1,1.0);
        return result;
}

vec3 normaltosurface(float s, float t, mat4 gx, mat4 gy, mat4 gz )
{
        vec4 svec1 = vec4(3*s*s,2*s,1.0,0.0);
        vec4 tvec1 = vec4(t*t*t,t*t,t,1.0);
        
	vec4 svec2 = vec4(s*s*s,s*s,s,1.0);
        vec4 tvec2 = vec4(3*t*t,2*t,1.0,0.0);
        

	vec4 xtemp1;
        vec4 ytemp1;
        vec4 ztemp1;
        xtemp1 = svec1*gx;
        ytemp1 = svec1*gy;
        ztemp1 = svec1*gz;
        float x1 = dot(xtemp1,tvec1);
        float y1 = dot(ytemp1,tvec1);
        float z1 = dot(ztemp1,tvec1);
        vec3 stangent = vec3(x1,y1,z1);
	normalize(stangent);
        xtemp1 = svec2*gx;
        ytemp1 = svec2*gy;
        ztemp1 = svec2*gz;
        x1 = dot(xtemp1,tvec2);
        y1 = dot(ytemp1,tvec2);
        z1 = dot(ztemp1,tvec2);
        vec3 ttangent = vec3(x1,y1,z1);
	normalize(ttangent);
        vec3 resnormal  = cross(stangent,ttangent);
        normalize(resnormal);
        return resnormal;
}

vec4 coloratpoint( vec3 normal, vec4 position)
{       vec4 finalcolor;
        vec4 innormal = vec4(normal.xyz,1.0);
        vec3 normalvec = normalize(  (gl_ModelViewMatrixInverseTranspose*innormal ).xyz );
        vec4 ptpos = gl_ModelViewMatrix* position;
        vec3 pv = normalize(ptpos.xyz);
        vec3 viewvec = -pv;
        vec4 lightpos =  gl_LightSource[0].position;
        vec3 lightvec = normalize ( lightpos.xyz*ptpos.w - ptpos.xyz*lightpos.w );            // Light Vector [in view space]
        vec3 reflvec = reflect( -lightvec, normalvec );                                       // Reflection Vector
        float diffuselight = max( dot( normalvec, lightvec), 0.0 );
        float specularlight=0;
        if(diffuselight > 0 ){
                specularlight = max( dot( reflvec, viewvec ), 0.0 );
                specularlight = pow( specularlight, gl_FrontMaterial.shininess  );
        }
        finalcolor = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
        finalcolor += gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse * diffuselight;
        finalcolor += gl_FrontMaterial.diffuse * gl_LightSource[1].diffuse * diffuselight;
        finalcolor += gl_FrontMaterial.specular * gl_LightSource[1].specular * specularlight;
        finalcolor += gl_FrontMaterial.specular * gl_LightSource[2].specular * specularlight;
        finalcolor += gl_FrontMaterial.specular * gl_LightSource[3].specular * specularlight;
        //finalcolor += gl_FrontMaterial.specular * gl_LightSource[4].specular * specularlight;
        //finalcolor += gl_FrontMaterial.specular * gl_LightSource[5].specular * specularlight;
	
        finalcolor += gl_FrontMaterial.specular * gl_LightSource[0].specular * specularlight;
        return finalcolor;
}






void main (void)
{	
	mat4 gx,gy,gz;
	gx = gx1;
	gy = gy1;
	gz = gz1;
        vec4 starray = 	gl_TexCoord[0];
	float s = starray[0];
	float t = starray[1];

	vec4 position;
	vec4 outcolor;
	position = floatposition(s,t,gx,gy,gz);
	vec3 outnormal = normaltosurface(s,t,gx,gy,gz);
	outcolor = coloratpoint(outnormal,position);	
	gl_FragColor = outcolor;
	




	

}

