/////////////////////////////////////////////////////////////////
//
//      Subdivision Geometry Shader Example
//      Author: Jag Mohan Singh
//      jagmohan [at] research [dot] iiit [dot] ac [dot] in
//      Center for Visual Information Technology
//      International Institute of Information Technology Hyderabad
//      http://cvit.iiit.ac.in/
//
/////////////////////////////////////////////////////////////////




#define GL_GLEXT_PROTOTYPES
#include <cstdlib>
#include <GL/glut.h>
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <iostream>
#include <cmath>
#include <sys/time.h>
#include <string>
using namespace std;
using std::string;
GLint loc1,loc2,loc3,loc4;
GLhandleARB vertexShader, fragmentShader, geometryShader, shader;

GLdouble  transformt[16] = {1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.847826, -0.0869565, -18.1358, 1.0};

float initpoint[3];
float angler = 0.0;
//Subdivision Parameter
int kdiv = 4;
int numpatches;
float **gx,**gy,**gz;

//global field of view
char *textFileRead(char *fn) {

        FILE *fp;
        char *content = NULL;

        int f,count;
        f = open(fn, O_RDONLY);

        count = lseek(f, 0, SEEK_END);

        close(f);

        if (fn != NULL) {
                fp = fopen(fn,"rt");

                if (fp != NULL) {
                        if (count > 0) {
                                content = (char *)malloc(sizeof(char) * (count+1));
                                count = fread(content,sizeof(char),count,fp);
                                content[count] = '\0';
                        }
                        fclose(fp);
                }
        }
        return content;
}


void printInfoLog(GLhandleARB obj, char * shadername){
        int infologLength = 0;
        int charsWritten = 0;
        char *infoLog;
        glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB,
                        &infologLength);
        if (infologLength > 0)
        {
                infoLog = (char *)malloc(infologLength);
                glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
                printf("%s\n",shadername);
                printf("%s\n",infoLog);
	
                free(infoLog);
        }
}




//Initializing the shaders
void initShaders() {

        char *vs,*fs,*gs;

        vertexShader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
        fragmentShader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
        geometryShader = glCreateShaderObjectARB(GL_GEOMETRY_SHADER_EXT);

        vs = textFileRead("vertexshader.glsl");
        fs = textFileRead("fragmentshader.glsl");
        gs = textFileRead("subdivisiongs.glsl");

        const char * ff = fs;
        const char * vv = vs;
        const char * gg = gs;

        glShaderSourceARB(vertexShader, 1, &vv,NULL);
        glShaderSourceARB(fragmentShader, 1, &ff,NULL);
        glShaderSourceARB(geometryShader, 1, &gg,NULL);

        free(vs);free(fs);free(gs);

        glCompileShaderARB(vertexShader);
        printInfoLog( vertexShader,"vertexshader.glsl" );
        glCompileShaderARB(fragmentShader);
        printInfoLog( fragmentShader, "fragmentshader.glsl" );
        glCompileShaderARB(geometryShader);
        printInfoLog( geometryShader, "subdivisionshader.glsl" );

        shader = glCreateProgramObjectARB();
        glAttachObjectARB( shader, vertexShader);
        glAttachObjectARB( shader, fragmentShader);
        glAttachObjectARB( shader, geometryShader);

	//Input type primitive and Output type primitive Last line is essential to specify the maximum number of output vertices
        glProgramParameteriEXT( shader, GL_GEOMETRY_INPUT_TYPE_EXT , GL_TRIANGLES );
        glProgramParameteriEXT( shader, GL_GEOMETRY_OUTPUT_TYPE_EXT , GL_TRIANGLES );
        glProgramParameteriEXT( shader, GL_GEOMETRY_VERTICES_OUT_EXT, 48);

        glLinkProgramARB( shader );
        glUseProgramObjectARB( shader );
	loc1 = glGetUniformLocation(shader,"gx1");
        loc2 = glGetUniformLocation(shader,"gy1");
        loc3 = glGetUniformLocation(shader,"gz1");

}






// Displaying the Patch
void displaypatch(int m)
{
	
	glPushMatrix();
	glMultMatrixd(transformt);
	glDepthFunc(GL_LESS);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	glRotatef(angler,0,1,0);
          

	glColor3f(1.0,0.875,0.575);



	glUniformMatrix4fv(loc1,1,0,gx[m]);
   	glUniformMatrix4fv(loc2,1,0,gy[m]);
   	glUniformMatrix4fv(loc3,1,0,gz[m]);




   float dx = 1.0/kdiv;
   float dy = 1.0/kdiv;



   glBegin(GL_TRIANGLES);
   for(int i=0;i<=kdiv-1;i++)
   for(int j=0;j<=kdiv-1;j++)
   {
	
   glVertex3f(j*dx,i*dy,1.0);
   glVertex3f((j+1)*dx,i*dy,1.0);
   glVertex3f(j*dx,(i+1)*dy,1.0);
   glVertex3f((j+1)*dx,i*dy,1.0);
   glVertex3f((j+1)*dx,(i+1)*dy,1.0);
   glVertex3f(j*dx,(i+1)*dy,1.0);
   
   }
   glEnd();
   glPopMatrix();
}

//Initializing OpenGL Variables
void setupGL( ) 
{
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glFrustum(-3.38394, 3.38394, -3.38394,3.38394, 13.67, 20.5098);


        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        GLfloat light0_position[] = {0.258604 ,-0.286189, -0.922616, 0 };
        glLightfv(GL_LIGHT0, GL_POSITION, light0_position);

	GLfloat mat_diffuse[4] = { 1, 1, 1, 1 };
        GLfloat mat_specular[4] = { 0.88, 0.88, 0.88, 0.88 };
	GLfloat mat_shininess[] = { 100 };
        GLfloat global_ambient[] = { 0.2, 0.2, 0.5, 1.0 };
        GLfloat light0_ambient[] = { 0.2, 0.2, 0.2, 0.2 };
        GLfloat light0_diffuse[] = { 0.85, 0.85, 0.85, 1.0 };
        GLfloat light1_diffuse[] = { -0.01, -0.01, -0.03, -0.03 };
        GLfloat light0_specular[] = { 0.85, 0.85, 0.85, 0.85 };
        GLfloat light_position[] = {0,0,0,1};


        glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, mat_diffuse);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
        glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
        glLightfv(GL_LIGHT0, GL_AMBIENT, light0_ambient);
        glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_diffuse);
        glLightfv(GL_LIGHT0, GL_SPECULAR, light0_specular);
        glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_diffuse);
        glLightfv(GL_LIGHT1, GL_SPECULAR, light0_specular);
        glLightfv(GL_LIGHT2, GL_SPECULAR, light0_specular);
        glLightfv(GL_LIGHT3, GL_SPECULAR, light0_specular);
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_LIGHT1);
        glEnable(GL_LIGHT2);
        glEnable(GL_LIGHT3);
        

        glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_COLOR_MATERIAL);
	

}



// Display Function
void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glUseProgramObjectARB( shader );
	for(int i=0;i<numpatches;i++)
	displaypatch(i);
	glutSwapBuffers();
}




// Keyboard Function
void keyboardfunc(unsigned char key, int x, int y)
{
	switch (key) {
             
		case 'a': angler+=10.0;
			  break;
		 
	
		case 'd': angler-=10.0;
			  break;

		case 's':
			 kdiv++;
			 break;
		
		case 'f':
			kdiv--;
			break;

	
		case 'q':
			exit(0);
	}
	glutPostRedisplay();
}


void usage(const char *myname)
{
	fprintf(stderr, "Usage: %s geometryvectorfile\n", myname);
	exit(1);
}



int main(int argc, char *argv[])
{

	initpoint[0] = -0.847826;
	initpoint[1] = 0.0869565;
	initpoint[2] =  1.04435;



	if( argc < 2 )
	usage(argv[0]);


	const char * gfilename = argv[1];
	FILE * gf = fopen(gfilename,"r");
	fscanf(gf,"%d",&numpatches);
	//Allocating memory to geometry vectors 
	gx = new float* [numpatches];
	gy = new float* [numpatches];
	gz = new float* [numpatches];
	
	for(int i=0;i<numpatches;i++)
	{
	  gx[i] = new float[16];
	  gy[i] = new float[16];
	  gz[i] = new float[16];
	}


	//Reading Geometry Vectors
	for(int i=0;i<numpatches;i++)
	{
	for(int j=0;j<16;j++)
		fscanf(gf,"%f",&gx[i][j]);
	  
	for(int j=0;j<16;j++)
		fscanf(gf,"%f",&gy[i][j]);

	for(int j=0;j<16;j++)
		fscanf(gf,"%f",&gz[i][j]);
	}

	glutInitWindowSize(512, 512);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	glutInit(&argc, argv);

	if (argc < 2)
		usage(argv[0]);

	glutCreateWindow("Subdivision Shader");
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboardfunc);
	setupGL();
	initShaders();
	glutMainLoop();
}

