/////////////////////////////////////////////////////////////////
//
//      Subdivision Geometry Shader Example
//      Author: Jag Mohan Singh
//      jagmohan [at] research [dot] iiit [dot] ac [dot] in
//      Center for Visual Information Technology
//      International Institute of Information Technology Hyderabad
//      http://cvit.iiit.ac.in/
//
/////////////////////////////////////////////////////////////////

#version 120
#extension GL_EXT_geometry_shader4 : enable 
// Extension required for geometry shader

//Output position from geometry shader
varying vec4 outposition;

//Input geometry vectors from CPU
uniform mat4 gx1;
uniform mat4 gy1;
uniform mat4 gz1;


vec4 floatposition( float s, float t, mat4 gx, mat4 gy, mat4 gz )
{

        float s3 = s*s*s;
        float s2 = s*s;

        float t3 = t*t*t;
        float t2 = t*t;

        vec4 svec = vec4(s3,s2,s,1.0);
        vec4 tvec = vec4(t3,t2,t,1.0);
        float x1 = dot(svec*gx,tvec);
        float y1 = dot(svec*gy,tvec);
        float z1 = dot(svec*gz,tvec);
        vec4 result = vec4(x1,y1,z1,1.0);
        return result;
}


vec3 normaltosurface(float s, float t, mat4 gx, mat4 gy, mat4 gz )
{
        vec4 svec1 = vec4(3*s*s,2*s,1.0,0.0);
        vec4 tvec1 = vec4(t*t*t,t*t,t,1.0);
        vec4 svec2 = vec4(s*s*s,s*s,s,0.0);
        vec4 tvec2 = vec4(3*t*t,2*t,1.0,0.0);
        vec4 xtemp1;
        vec4 ytemp1;
        vec4 ztemp1;
        xtemp1 = svec1*gx;
        ytemp1 = svec1*gy;
        ztemp1 = svec1*gz;
        float x1 = dot(xtemp1,tvec1);
        float y1 = dot(ytemp1,tvec1);
        float z1 = dot(ztemp1,tvec1);
        vec3 stangent = vec3(x1,y1,z1);
        xtemp1 = svec2*gx;
        ytemp1 = svec2*gy;
        ztemp1 = svec2*gz;
        x1 = dot(xtemp1,tvec2);
        y1 = dot(ytemp1,tvec2);
        z1 = dot(ztemp1,tvec2);
        vec3 ttangent = vec3(x1,y1,z1);
        vec3 resnormal  = cross(stangent,ttangent);
        normalize(resnormal);
        return resnormal;
}

vec4 coloratpoint( vec3 normal, vec4 position)
{       vec4 finalcolor;
        vec3 normalvec = normalize(  gl_NormalMatrix*normal.xyz  );
        vec4 ptpos = gl_ModelViewMatrix* position;
        vec3 pv = normalize(ptpos.xyz);
        vec3 viewvec = -pv;
        vec4 lightpos =  gl_LightSource[0].position;
        vec3 lightvec = normalize ( lightpos.xyz*ptpos.w - ptpos.xyz*lightpos.w );            // Light Vector [in view space]
        vec3 reflvec = reflect( -lightvec, normalvec );                                       // Reflection Vector
        float diffuselight = max( dot( normalvec, lightvec), 0.0 );
        float specularlight=0;
        if(diffuselight > 0 ){
                specularlight = max( dot( reflvec, viewvec ), 0.0 );
                specularlight = pow( specularlight, gl_FrontMaterial.shininess  );
        }
        finalcolor = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
        finalcolor += gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse * diffuselight;
        finalcolor += gl_FrontMaterial.specular * gl_LightSource[0].specular * specularlight;
        return finalcolor;
}


void main(void)
{


	//Incoming values of (s,t) parameters from the input triangle
	vec4 a = gl_PositionIn[0];
	vec4 b = gl_PositionIn[1];
	vec4 c = gl_PositionIn[2];
	

	
	//Step size of the parameter values for two levels of subdivision
	
	vec4 dx = (b-a)/4.0;
	vec4 dy = (c-a)/4.0;
	
	vec4 vertices[25];
	float m,n;

	
		
	//These are the parameter values of the vertices which are created
	//after two levels of subdivision. This can be seen as dividing
	//the triangle along each parameter value into four equal parts
	//thus creating 16 triangles.	
	for(int i=0;i<5;i++)
	for(int j=0;j<5;j++)
	vertices[5*i+j] = a+dx*j+dy*i;



	mat4 gx,gy,gz;
	
	gx = gx1;
	gy = gy1;
	gz = gz1;
	







	 	
	vec3 outnormal;
        vec4 outcolor;

	//Two Levels of Subdivision create 16 triangles and 25 vertices are 
	//needed to represent these.
	

	
	
	//The triangles can be seen as strips of four rows of triangles
	
	//The first row of triangles created
	

	//Inorder to create newtriangles three vertices are generated and their end is signified by EndVertex
	//The end of triangle is signified by EndPrimitive
	for(int i=0;i<3;i++)
	{
	gl_TexCoord[0] = vertices[i];
        outposition = floatposition(vertices[i][0],vertices[i][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i][0],vertices[i][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+1];
        outposition = floatposition(vertices[i+1][0],vertices[i+1][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+1][0],vertices[i+1][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+5];
        outposition = floatposition(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();


	gl_TexCoord[0] = vertices[i+1];
        outposition = floatposition(vertices[i+1][0],vertices[i+1][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+1][0],vertices[i+1][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+6];
        outposition = floatposition(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+5];
        outposition = floatposition(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();
	}


	//Last triangle of first row
	gl_TexCoord[0] = vertices[3];
        outposition = floatposition(vertices[3][0],vertices[3][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[3][0],vertices[3][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[4];
        outposition = floatposition(vertices[4][0],vertices[4][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[4][0],vertices[4][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[8];
        outposition = floatposition(vertices[8][0],vertices[8][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[8][0],vertices[8][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();


	//Second Row of Triangles created
	for(int i=0;i<2;i++)
	{
	gl_TexCoord[0] = vertices[i+5];
        outposition = floatposition(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+5][0],vertices[i+5][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+6];
        outposition = floatposition(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+10];
        outposition = floatposition(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();


	gl_TexCoord[0] = vertices[i+6];
        outposition = floatposition(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+6][0],vertices[i+6][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();


        gl_TexCoord[0] = vertices[i+11];
        outposition = floatposition(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();


        gl_TexCoord[0] = vertices[i+10];
        outposition = floatposition(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();
	}


	//Last triangle of second row
	gl_TexCoord[0] = vertices[7];
        outposition = floatposition(vertices[7][0],vertices[7][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[7][0],vertices[7][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[8];
        outposition = floatposition(vertices[8][0],vertices[8][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[8][0],vertices[8][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[12];
        outposition = floatposition(vertices[12][0],vertices[12][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[12][0],vertices[12][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
       
	
	//Third Row of triangles
	for(int i=0;i<1;i++)
	{
	gl_TexCoord[0] = vertices[i+10];
        outposition = floatposition(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+10][0],vertices[i+10][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+11];
        outposition = floatposition(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[i+15];
        outposition = floatposition(vertices[i+15][0],vertices[i+15][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+15][0],vertices[i+15][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();


	gl_TexCoord[0] = vertices[i+11];
        outposition = floatposition(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+11][0],vertices[i+11][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();


        gl_TexCoord[0] = vertices[i+16];
        outposition = floatposition(vertices[i+16][0],vertices[i+16][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+16][0],vertices[i+16][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();


        gl_TexCoord[0] = vertices[i+15];
        outposition = floatposition(vertices[i+15][0],vertices[i+15][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[i+15][0],vertices[i+15][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        EndPrimitive();
	}

	
	//Last triangle of third row
	gl_TexCoord[0] = vertices[11];
        outposition = floatposition(vertices[11][0],vertices[11][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[11][0],vertices[11][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[12];
        outposition = floatposition(vertices[12][0],vertices[12][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[12][0],vertices[12][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[16];
        outposition = floatposition(vertices[16][0],vertices[16][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[16][0],vertices[16][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
       

	//Fourth Row which has single triangle
	gl_TexCoord[0] = vertices[15];
        outposition = floatposition(vertices[15][0],vertices[15][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[15][0],vertices[15][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[16];
        outposition = floatposition(vertices[16][0],vertices[16][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[16][0],vertices[16][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
        gl_TexCoord[0] = vertices[20];
        outposition = floatposition(vertices[20][0],vertices[20][1],gx,gy,gz);
        outnormal = normaltosurface(vertices[20][0],vertices[20][1],gx,gy,gz);
        outcolor = coloratpoint(outnormal,outposition);
        gl_FrontColor = outcolor;
        gl_Position = gl_ModelViewProjectionMatrix*outposition;
        EmitVertex();
       
	
	
	

}
